import { Component, EventEmitter, Input, Output } from '@angular/core';
import { EmployeesForAQualificationDTO, QualificationGetDTO } from '../../models/skill';
import { FormControl, FormGroup, ReactiveFormsModule, Validators } from '@angular/forms';
import { SkillService } from '../../service/skill.service';
import { EmployeeService } from '../../service/employee.service';
import { concatMap, every, from, lastValueFrom, Observable, of, switchMap, take, tap } from 'rxjs';
import { EmployeeNameDataDTO, EmployeeResponseDTO } from '../../models/mitarbeiter';
import { AsyncPipe } from '@angular/common';

@Component({
  selector: 'app-qualifikation-form',
  standalone: true,
  imports: [ReactiveFormsModule, AsyncPipe],
  templateUrl: './qualifikation-form.component.html',
  styleUrl: './qualifikation-form.component.css'
})
export class QualifikationFormComponent {
  @Input() skill!: QualificationGetDTO;
  @Output() skillChange = new EventEmitter<QualificationGetDTO>();

  public skillForm!: FormGroup;
  public addableEmployees: Array<EmployeeResponseDTO> = [];
  public addedEmployees: Array<EmployeeNameDataDTO> = [];

  constructor(private skillService: SkillService, private employeeService: EmployeeService) { }

  setUpForm() {
    this.skillForm = new FormGroup({
      name: new FormControl(this.skill.skill, Validators.required),
      newEmployee: new FormControl(),
    });
  }


  addEmployee() {
    console.log("called");
    const employeeId = Number(this.skillForm.get("newEmployee")?.value);
    const employee = this.addableEmployees.find(emp => emp.id === employeeId);

    if (employee) {
      this.addableEmployees = this.addableEmployees.filter(emp => emp.id !== employeeId);

      this.addedEmployees.push(employee);
    }
  }

  removeEmployee(employeeId: number) {
    const employee = this.addedEmployees.find(emp => emp.id === employeeId);

    if (employee) {
      this.addedEmployees = this.addedEmployees.filter(emp => emp.id !== employeeId);

      this.employeeService.getEmployeeById(employee.id).subscribe(employeeDto => {
        this.addableEmployees.push(employeeDto);
      });
    }
  }

  updateEmployeeLists() {
    if (this.skill.id != -1) {
      this.employeeService.getAllEmployees().subscribe(employees => {
        this.addableEmployees = employees;

        this.skillService.getEmployeesBySkill(this.skill.id).subscribe(addedEmployeesResponse => {
          this.addedEmployees = addedEmployeesResponse.employees;

          this.addableEmployees = this.addableEmployees.filter(employee => {
            return !this.addedEmployees.some(added => added.id === employee.id);
          });
        });
      });
    }
  }

  ngOnChanges(): void {
    this.setUpForm();
    this.updateEmployeeLists();
  }

  ngOnInit() {
    this.setUpForm();
    this.updateEmployeeLists();
  }
}
